var Cards = {
	Collection: new Array(100),
	CardData: new Array(100),
	Stats: {
		Win: 0,
		Lose: 0,
		Draw: 0
	},
	CommonImages: {
		Icons: {
			None: LoadImage("card-icon-none.png"),
			Monster: [LoadImage("card-icon-monster.png"), LoadImage("card-icon-monster-dark.png")],
			QMark: [LoadImage("card-icon-qmark.png"), LoadImage("card-icon-qmark-dark.png")]
		},
		CardSelection: LoadImage("card-selection.png"),
		CardReverse: LoadImage("card-reverse.png"),
		CardBack: LoadImage("cardback1.png"),
		ScoreSep: LoadImage("card-ss.png"),
		Arrows: [
			LoadImage("card-a-u.png"),
			LoadImage("card-a-ur.png"),
			LoadImage("card-a-r.png"),
			LoadImage("card-a-br.png"),
			LoadImage("card-a-b.png"),
			LoadImage("card-a-bl.png"),
			LoadImage("card-a-l.png"),
			LoadImage("card-a-ul.png")
		]
	},
	ArrowPositions: [
		[17, 1],
		[34, 2],
		[36, 21],
		[34, 43],
		[17, 45],
		[2, 43],
		[1, 21],
		[2, 2]
	],
	LoadCollection: function(collection)
	{
		this.Collection = new Array(100);
		for (var i = 0; i < 100; i++)
		{
			this.Collection[i] = [];
			if (!collection[i]) collection[i] = [];
			for (var j = 0; j < collection[i].length; j++)
			{
				this.Collection[i].push(this.LoadCard(i, collection[i][j]));
			}
		}
	},
	
	LoadCard: function(card, stats)
	{
		if (!this.CardData[card]) this.CardData[card] = Data.Load("cards/" + PadLeft(card, 3, "0") + ".cardData");
		return new Card(this.CardData[card], stats);
	},
	
	PlayGame: function(opponent)
	{
		Map.FadeOut();
		Screen.NeverRenderMap = true;
		Map.HUD.Visible = false;
		var game = new CardGame();
		var picker = new CardPicker();
		Screen.Attach(picker);
		picker.Init();
		Screen.Input(picker);
		while (!picker.Completed)
		{
			Screen.Frame();
		}
		var fo = new FadeOut();
		Screen.Attach(fo);
		while (!fo.Completed) Screen.Frame();
		Screen.Detach(fo);
		var cards = picker.SelectedCards;
		for (var i = 0; i < cards.length; i++)
		{
			Screen.Detach(cards[i]);
			cards[i] = cards[i].Card;
		}
		Screen.Detach(picker.cs);
		game.SetCards(cards);
		Screen.Detach(picker);
		picker.CleanDetach();
		Screen.Attach(game);
		Screen.AttachPostRenderEffect(new FadeIn()).Monitor = true;
		game.Init();
		Screen.Input(game);
		while (!game.Completed)
		{
			Screen.Frame();
		}
		Screen.Detach(game);
		Screen.NeverRenderMap = false;
		Map.FadeIn();
	},
	
	DrawAt: function(card, x, y, na)
	{
		Cards.CommonImages.CardBack.blit(x, y);
		card.Image.blit(x, y);
		if (na) return;
		for (var i = 0; i < card.Arrows.length; i++)
		{
			if (card.Arrows[i] == 1)
			{
				Cards.CommonImages.Arrows[i].blit(x + Cards.ArrowPositions[i][0], y + Cards.ArrowPositions[i][1]);
			}
		}
		var a = "0123456789ABCDEF";
		var s = a.substr(card.Stats[0],1) + "PMAX".substr(card.Stats[1],1) + a.substr(card.Stats[2],1) + a.substr(card.Stats[3],1);
		Font.Card.drawText(x + 7, y + 34, s);
	}
}

function Card(data, stats)
{
	this.Data = data;
	this.Image = LoadImage("cards/" + data.Image);
	this.Icon = Cards.CommonImages.Icons[data.Icon];
	this.ID = parseInt(data.ID);
	this.FlipState = 0;
	if (!stats)
	{
		this.Stats = [
			Random(data.MinStats[0], data.MaxStats[0]), 
			Random(data.MinStats[1], data.MaxStats[1]), 
			Random(data.MinStats[2], data.MaxStats[2]), 
			Random(data.MinStats[3], data.MaxStats[3])
		];
		this.Arrows = new Array(8);
		for (var i = 0; i < 8; i++)
		{
			if (data.MinArrows[i])
			{
				this.Arrows[i] = true;
			}
			else if (data.MaxArrows[i])
			{
				this.Arrows[i] = Random(0,1) == 1 ? true : false;
				}
			else
			{
				this.Arrows[i] = false;
			}
		}
	}
	else
	{
		this.Stats = stats.Stats;
		this.Arrows = stats.Arrows;
	}
}

function CardGame()
{
	this.Completed = false;
	this.Background = LoadImage("cardbg.png");
	this.Mode = "i";
	this.PlayerCards = [
		[],
		[]
	];
	this.ScreenCards = [
		[],
		[]
	];
	
	this.Init = function()
	{
		// do things...
		for (var i = 0; i < this.PlayerCards[1].length; i++)
		{
			var sc = new CardObject(this.PlayerCards[1][i], 266, 12 + 41 * i);
			this.PlayerCards[1][i].FlipState = 0;
			this.ScreenCards[1].push(sc);
			var sc2 = new CardObject(false, 13, 12 + 20 * i);
			Screen.Attach(sc);
			Screen.Attach(sc2);
			Screen.Frames(5);
		}
	}
	this.Key = function(key)
	{
	}
	this.Render = function()
	{
		this.Background.blit(51, 0);
		Cards.CommonImages.ScoreSep.blit(13, 188);
	}
	this.SetCards = function(cards)
	{
		this.PlayerCards[1] = cards;
	}
}
function CardPicker()
{
	this.Completed = false;
	this.Background = LoadImage("cardbg2.png");
	this.MoreCards = LoadImage("morecards.png");
	this.CardMenu = false;
	this.Selected = 0;
	this.selx = 0;
	this.sely = 0;
	this.SelectedCards = [];
	this.Cursor = new Cursor(5, 44);
	this.sels = {};
	this.selc = CreateColor(255, 0, 0);
	this.Init = function()
	{
		Screen.Attach(this.Cursor);
	}
	this.CleanDetach = function()
	{
		for (var i = 0; i < this.SelectedCards.length; i++)
		{
			Screen.Detach(this.SelectedCards[i]);
		}
		Screen.Detach(this.Cursor);
	}
	this.Render = function()
	{
		if (this.Completed) return;
		this.Background.blit(0, 0);
		Cards.CommonImages.CardSelection.blit(92, 8);
		Window.Draw(180, 50, 76, 16*3);
		Font.Message.drawText(184, 54, "Win");
		Font.Message.drawText(184, 70, "Lose");
		Font.Message.drawText(184, 86, "Draw");
		var y = 54;
		for (var i in Cards.Stats)
		{
			Font.Message.drawText(254 - Font.Message.getStringWidth(Cards.Stats[i]), y, Cards.Stats[i]);
			y += 16;
		}
		Window.Draw(16, 40, 150, 150);
		for (var i = 0; i < Cards.Collection.length; i++)
		{
			var y = i % 10;
			var x = (i - y) / 10;
			if (Cards.Collection[i].length == 1)
			{
				var icon = Cards.Collection[i][0].Icon[0];
				icon.blit(17 + x * 15, 41 + y * 15);
			}
			else if (Cards.Collection[i].length > 1)
			{
				var icon = Cards.Collection[i][0].Icon[1];
				icon.blit(17 + x * 15, 41 + y * 15);
				Font.Small.drawText(16 + (x + 1) * 15 - Font.Small.getStringWidth(Cards.Collection[i].length), 34 + (y + 1) * 15, Cards.Collection[i].length);
			}
			else
			{
				var icon = Cards.CommonImages.Icons.None;
				icon.blit(17 + x * 15, 41 + y * 15);
			}

		}
		var selid = this.selx * 10 + this.sely;
		if (Cards.Collection[selid].length)
		{
			if (!this.sels[selid] || this.sels[selid] >= Cards.Collection[selid].length) this.sels[selid] = 0;
			if (Cards.Collection[selid].length > 1)
			{
				if (this.Scrolled)
				{
					Cards.DrawAt(Cards.Collection[selid][this.sels[selid]], 202, 128, true);
					Cards.DrawAt(Cards.Collection[selid][this.sels[selid]], 200, 128, true);	
					this.Scrolled = false;
				}
				else
				{
					Cards.DrawAt(Cards.Collection[selid][this.sels[selid]], 205, 128, true);
					Cards.DrawAt(Cards.Collection[selid][this.sels[selid]], 201, 128, true);
				}
			}
			Cards.DrawAt(Cards.Collection[selid][this.sels[selid]], 197, 128);
			if (Cards.Collection[selid].length > 1)
			{
				var st = (this.sels[selid] + 1) + "/" + Cards.Collection[selid].length;
				Font.Message.drawText(218 - Font.Message.getStringWidth(st) / 2, 183, st);
			}
			Font.Message.drawText(218 - Font.Message.getStringWidth(Cards.Collection[selid][0].Data.Name) / 2, 203, Cards.Collection[selid][0].Data.Name);
		}
	}
	this.Update = function()
	{
		
	}
	this.Key = function(key)
	{
		if (this.SelectedCards.length == 5) return;
		var moved = false;
		var selid = this.selx * 10 + this.sely;
		switch (key)
		{
			case Keys.Up:
				this.sely--;
				if (this.sely < 0) this.sely = 9;
				moved = true;
				break;
			case Keys.Down:
				this.sely++;
				if (this.sely > 9) this.sely = 0;
				moved = true;
				break;
			case Keys.Left:
				this.selx--;
				if (this.selx < 0) this.selx = 9;
				moved = true;
				break;
			case Keys.Right:
				this.selx++;
				if (this.selx > 9) this.selx = 0;
				moved = true;
				break;
			case Keys.ScrollLeft:
				if (Cards.Collection[selid].length > 1)
				{
					this.sels[selid]--;
					if (this.sels[selid] == -1) this.sels[selid] = Cards.Collection[selid].length - 1;
					this.Scrolled = true;
				}
				break;
			case Keys.ScrollRight:
				if (Cards.Collection[selid].length > 1)
				{
					this.sels[selid]++;
					if (this.sels[selid] == Cards.Collection[selid].length) this.sels[selid] = 0;
					this.Scrolled = true;
				}
				break;
			case Keys.Accept:
				if (Cards.Collection[selid].length)
				{
					var sc = new CardObject(Cards.Collection[selid][this.sels[selid]], 266, 12 + 41 * this.SelectedCards.length);
					this.SelectedCards.push(sc);
					Screen.Attach(sc);
					Cards.Collection[selid].splice(this.sels[selid], 1);
					if (this.sels[selid] == Cards.Collection[selid].length)
					{
						this.sels[selid]--;
					}
					this.Scrolled = true;
				}
				if (this.SelectedCards.length == 5)
				{
					while (!this.SelectedCards[4].Completed) { Screen.Frame(); }
					var cs = new ColorSlide(Color.Black50);
					this.cs = cs;
					Screen.Attach(cs);
					var msg = new TextBox(100, 90, 120, 16, "Are you sure?");
					Screen.Attach(msg);
					var choiceMenu = new ChoiceMenu(130, 120, 60, 16);
					choiceMenu.DrawMode = Window.Draw;
					choiceMenu.AddItem("Yes", true);
					choiceMenu.AddItem("No", true);
					choiceMenu.FigureHeight();
					Screen.Attach(choiceMenu);
					choiceMenu.Init();
					Screen.Input(choiceMenu);
					while (!choiceMenu.Completed)
					{
						Screen.Frame();
					}

					if (!choiceMenu.Selected)
					{
						Screen.Detach(choiceMenu);
						Screen.Detach(msg);
						this.Completed = true;
						break;
					}
					else {
						// fall through...
						Screen.Detach(choiceMenu);
						Screen.Detach(msg);
						Screen.Detach(this.cs);
						Screen.Attach(new ColorSlideOut(Color.Black50)).Monitor = true;
					}
				}
				else {
					break;
				}
			case Keys.Cancel:
				if (this.SelectedCards.length)
				{
					var card = this.SelectedCards.pop();
					Screen.Detach(card);
					Cards.Collection[card.Card.ID - 1].push(card.Card);
					this.sels[card.Card.ID - 1] = Cards.Collection[card.Card.ID - 1].length - 1;
				}
				break;
		}
		if (moved)
		{
			this.Cursor.MoveTo(5 + this.selx * 15, 44 + this.sely * 15, 5);
		}
	}
}

function CardObject(card, x, y)
{
	this.Card = card;
	this.FinX = x;
	this.FinY = y;
	this.x = x;
	this.y = 240;
	this.yps = 16;
	this.Render = function()
	{
		if (this.Card) Cards.DrawAt(this.Card, this.x, this.y);
		else Cards.CommonImages.CardReverse.blit(this.x, this.y);
	}
	this.Update = function()
	{
		this.y = Math.max(this.FinY, this.y - this.yps);
		if (this.y == this.FinY) this.Completed = true;
	}
}