var Characters = {
	List: {},
	
	Load: function(chars)
	{
		for (var i = 0; i < chars.length; i++)
		{
			this.List[chars[i]] = new Character(Data.Load("chars/" + chars[i] + ".charData"));
		}
	}
}
function Character(data)
{
	this.HP = data.HP;
	this.MaxHP = data.HP;
	this.MP = data.MP;
	this.MaxMP = data.MP;
	this.DisplayHP = this.HP;
	this.DisplayMP = this.MP;
	this.Job = data.Job;
	this.Equipment = { Weapon: { Attack: 70 }, Armour: { EvadePcnt: 10, Defense: 50 } };
	this.CurrentCommands = [];
	for (var i = 0; i < data.CurrentCommands.length; i++)
	{
		this.CurrentCommands.push(new Command(Data.Load("commands/" + data.CurrentCommands[i] + ".commandData")));
	}
	this.Abilities = [];
	for (var i = 0; i < data.Abilities.length; i++)
	{
		this.Abilities.push(new Attack(Data.Load("attacks/" + data.Abilities[i] + ".attackData")));
	}
	this.Sprite = data.Sprite;
	this.Level = data.Level;
	this.Name = data.Name;
	this.BaseStats = data.BaseStats;
	this.Spriteset = LoadSpriteset(this.Sprite + "-" + this.Job + ".rss");
	this.ATB = 0;
	this.currentX = 0;
	this.AttackType = new Attack(Data.Load("attacks/Sword.attackData"));
	this.currentY = 0;
	this.Status = {
		Haste: false,
		Slow: false,
		Break: false
	}
	this.Destroy = false;
	this.x = 0;
	this.Selected = false;
	this.SelectDir = 2;
	this.SelectAlpha = 64;
	this.SelectMask = CreateColor(255, 255, 255);
	
	this.WaitingForAction = false;
	this.y = 0;
	this.Facing = "west";
	this.MoveStart = 0;
	this.MoveX = 0;
	this.MoveY = 0;
	this.Moving = false;
	this.AnimFrame = 0;
	this.AnimDelay = 0;
	this.Direction = Scene._FindSpritesetDirection(this.Spriteset, "west");

	this.GetBaseX = function()
	{
		return this.currentX + (this.Spriteset.base.x1 + this.Spriteset.base.x2) / 2;
	}
	this.GetBaseY = function()
	{
		return this.currentY + (this.Spriteset.base.y1 + this.Spriteset.base.y2) / 2;
	}

	this.Render = function()
	{
		this.Spriteset.images[this.Direction.frames[this.AnimFrame].index].blit(this.currentX, this.currentY);
		if (this.Selected)
		{
			Common.Selector.blitMask(this.currentX - 4, this.currentY + 4, this.SelectMask);
		}
	}
	this.SetDirection = function(dir)
	{
		this.Direction = Scene._FindSpritesetDirection(this.Spriteset, dir);
		this.AnimFrame = 0;
	}
	this.SetPose = function(pose)
	{
		if (pose)
			this.SetDirection(this.Facing + "-" + pose);
		else
			this.SetDirection(this.Facing);
	}
	this.AnimatePose = function(pose)
	{
		this.AnimateDirection(this.Facing + "-" + pose);
	}
	this.AnimateDirection = function(dir)
	{
		var o = this.Direction.name;
		this.SetDirection(dir);
		var sdirection = this.Direction;
		for (var i = 0; i < sdirection.frames.length; i++)
		{
			this.AnimFrame = i;
			Screen.Frames(sdirection.frames[i].delay);
		}
		this.SetDirection(o);
	}
	this.MoveTo = function(x, y, frames)
	{
		if (frames)
		{
			this.MoveStart = GetTime();
			this.MoveX = (x - this.currentX) / frames;
			this.MoveY = (y - this.currentY) / frames;
			this.Moving = true;
			this.MoveFrames = frames;
			this.MoveFrameI = 0;
			this.x = this.currentX;
			this.y = this.currentY;
		}
		else
		{
			this.currentX = x;
			this.currentY = y;
		}
	}
	this.Update = function()
	{
		/*this.AnimDelay++;
		if (this.AnimDelay == this.Direction.frames[this.AnimFrame].delay)
		{
			this.AnimFrame++;
			this.AnimDelay = 0;
			if (this.AnimFrame == this.Direction.frames.length)
			{
				this.AnimFrame = 0;
			}
		}*/
		if (this.Selected)
		{
			if (this.SelectMask.alpha + this.SelectDir >= 255)
			{
				this.SelectMask.alpha = 255;
				this.SelectDir = -2;
			}
			else if (this.SelectMask.alpha + this.SelectDir <= 128)
			{
				this.SelectMask.alpha = 128;
				this.SelectDir = 2;
			}
			else
			{
				this.SelectMask.alpha += this.SelectDir;
			}
		}
		if (this.Moving)
		{
			this.currentX += this.MoveX;
			this.currentY += this.MoveY;
			this.MoveFrameI++;
			if (this.MoveFrameI == this.MoveFrames)
			{
				this.Moving = false;
			}
		}
		if (this.DisplayHP != this.HP)
		{
			if (this.DisplayHP < this.HP) this.DisplayHP = Math.min(this.DisplayHP + 25, this.HP);
			if (this.DisplayHP > this.HP) this.DisplayHP = Math.max(this.DisplayHP - 25, this.HP);
		}
		if (this.DisplayMP != this.MP)
		{
			if (this.DisplayMP < this.MP) this.DisplayMP = Math.min(this.DisplayMP + 10, this.MP);
			if (this.DisplayMP > this.MP) this.DisplayMP = Math.max(this.DisplayMP - 10, this.MP);
		}
	}
	
	this.ResetATB = function()
	{
	/*
	i)   Let N1 = 120 - Agility + (Equipment Weight/8). Minimum value = 1.
			   For Monsters, use Speed instead of Agility, Equipment Weight = 0.
			   
	ii)  If they have Haste status, N1 = N1 / 2. Minimum value = 1.
	
	iii) If they have Slow status, N1 = N1 * 2. Maximum value = 255.
         Note that Haste and Slow are mutually exclusive statuses. You can only 
         have one or the other, but never both at the same time.
         
	iv)  ATB = 255 - N1
	*/
		var n1 = Math.max(120 - this.GetStat('Agility') + Math.floor(this.GetEquipmentWeight() / 8), 1);
		if (this.Status.Haste) n1 = Math.floor(n1 / 2);
		else if (this.Status.Slow) n1 = n1 * 2;
		this.ATB = 255 - n1;
				this.WaitingForAction = false;
	}
	
	this.GetStat = function(name)
	{
		return this.BaseStats[name] + Jobs[this.Job].Stats[name];
	}
	
	this.ChangeRow = function()
	{
		if (this.BackRow)
		{
			this.SetPose(false);
			this.MoveTo(this.currentX - 24, this.currentY, 12);
			this.AnimatePose("moving");
			//Screen.Frames(5);
		}
		else
		{
			this.Facing = "east";
			this.SetPose(false);
			this.MoveTo(this.currentX + 24, this.currentY, 12);
			this.AnimatePose("moving");
			//Screen.Frames(5);
			this.Facing = "west";
			this.SetPose(false);		
		}
		this.BackRow = !this.BackRow;
	}
	
	this.GetEquipmentStat = function(stat)
	{
		var n = 0;
		for (var i in this.Equipment)
		{
			if (this.Equipment[i][stat])
				n += this.Equipment[i][stat];
		}
		return n;
	}
	
	this.Action = function()
	{
		
		this.HasSelectedAction = false;
		this.ActionCanceled = false;
		this.Selected = true;
		var menu = new ChoiceMenu(40, 154, 70, 72);
		menu.AllowCancel = true;
		
		for (var i = 0; i < this.CurrentCommands.length; i++)
		{
			if (this.CurrentCommands[i].Type == "Builtin")
			{
				if (this.CurrentCommands[i].Builtin == "Fight")
				{
					menu.AddItem(this.CurrentCommands[i].Name, true, ActionTargetSelect, this.AttackType);
				}
			}
			else if (this.CurrentCommands[i].Type == "Action")
			{
				menu.AddItem(this.CurrentCommands[i].Name, true, ActionTargetSelect, this.CurrentCommands[i].Action);
			}
			else if (this.CurrentCommands[i].Type == "AbilityList")
			{
				var abilities = this.CurrentCommands[i].GetAbilities(this);
				// create a menu here
				var items = [];
				var nmenu = {
					x: 60, y: 154, w: 100, h: 72, AllowCancel: true};
				for (var j = 0; j < abilities.length; j++)
				{
					items.push(
						{
							Text: abilities[j].Name,
							Enabled: (this.MP >= abilities[j].MPCost), 
							RetFunc: ActionTargetSelect,
							RetFuncParam: abilities[j],
							EndText: abilities[j].MPCost
						}
					);
				}
				nmenu.Items = items;
				menu.AddItem(this.CurrentCommands[i].Name, (abilities.length ? true : false), ChoiceMenuCreator, nmenu, true);
			}
		}
		
		menu.LeftMenu = {NoNew: true, RetFunc: ChoiceMenuCreator, RetFuncParam: {
			x: 20, y: 154, w: 40, h: 24, RightCancels: true, AllowCancel: true,
			Items: [
				{ Text: "Row", Enabled: true, RetFunc: PerformAction, RetFuncParam: {Action: new ChangeRow(), Targets: [this]} }
			]
		}};
		
		Screen.Attach(menu);
		Screen.Input(menu);
		menu.Init();
		this.ActionMenu = menu;
	}
	this.UpdateAction = function()
	{
		if (this.ActionMenu.Completed)
		{
			Screen.Detach(this.ActionMenu);
			this.SelectedAction = this.ActionMenu.Return;
			if (this.ActionMenu.Canceled) this.ActionCanceled = true;
			else
			{
				this.ActionCanceled = false;
				this.SetPose("ready");
				Battle.CurrentBattle.QueueAction(this);
			}
			this.HasSelectedAction = true;
			this.Selected = false;
		}
	}
	this.EndBattle = function()
	{
		Screen.Detach(this.ActionMenu);
		this.Selected = false;
		if (!this.Status.Dead) this.SetPose(false);
	}
	this.TakeDamage = function(damage)
	{
		this.HP -= damage;
		this.Check();
	}
	this.StartBattle = function()
	{
		if (this.Status.Dead) this.SetPose('dead');
		else
		{
			this.SetPose(false);
			this.ResetATB();
		}
		this.DisplayHP = this.HP;
	}
	this.Check = function()
	{
		if (this.HP <= 0)
		{
			this.HP = 0;
			this.SetPose("dead");
			this.ATB = 0;
			this.Status.Dead = true;
			if (this.Selected)
			{
				this.Selected = false;
				this.ActionMenu.CleanDetach();
				Screen.Detach(this.ActionMenu);
				Battle.CurrentBattle.CancelAction(this);
			}
			if (this.HasSelectedAction)
			{
				Battle.CurrentBattle.DequeueAction(this);
				this.WaitingForAction = false;
			}
			//Battle.CurrentBattle.CheckConditions();
		}
	}
	this.ExecuteAction = function()
	{
		if (Battle.CurrentBattle.AlmostCompleted) return;
		Battle.CurrentBattle.Wait = true;
		Screen.Frames(10);
		if (this.SelectedAction[0].MPCost)
		{
			this.MP -= this.SelectedAction[0].MPCost;
		}
		if (this.SelectedAction[0].ShowName)
		{
			Screen.Attach(new BattleAttackName(this.SelectedAction[0].Name)).Monitor = true;
		}
			if (this.SelectedAction[0].LeadUp)
			{
				this.SetPose(false);
				this.MoveTo(this.currentX - 24, this.currentY, 12);
				this.AnimatePose("moving");
				Screen.Frames(10);
			}

		if (this.SelectedAction.length)
		{
			if (this.SelectedAction[1].length > 1)
			{
				// get rid of dead targets
				actualTargets = [];
				for (var i = 0; i < this.SelectedAction[1].length; i++)
				{
					if (!this.SelectedAction[1][i].Status.Dead) actualTargets.push(this.SelectedAction[1][i]);
				}
			}
			else
			{
				if (this.SelectedAction[1][0].Status.Dead)
				{
					// is it a monster? choose another one
					if (this.SelectedAction[1][0] instanceof Monster)
					{
						actualTargets = [Battle.CurrentBattle.Monsters[0]];
					}
					else
					{
						// otherwise, its a character, and you're a dumbass
						actualTargets = this.SelectedAction[1];
					}
				}
				else
				{
					actualTargets = this.SelectedAction[1];
				}
			}
			this.SelectedAction[0].Invoke(this, actualTargets);
		}
		if (!this.Status.Dead)
		{
			if (this.SelectedAction[0].LeadUp)
			{
				this.Facing = "east";
				this.MoveTo(this.currentX + 24, this.currentY, 12);
				this.AnimatePose("moving");
				this.Facing = "west";
				this.SetPose(false);
			}
			this.ResetATB();
		}
		else
		{
			this.MoveTo(this.currentX + 24, this.currentY, 0);
		}
		Battle.CurrentBattle.Wait = false;
	}
	
	this.BattleUpdate = function()
	{
		if (!this.Status.Dead && !this.WaitingForAction)
		{
			this.ATB += 0.5;
			if (this.ATB == 255)
			{
				this.WaitingForAction = true;
				Battle.CurrentBattle.RequestAction(this);
			}
		}
	}
	
	this.GetEquipmentWeight = function()
	{
		// DO THIS PROPERLY IDIOT
		return 20;
	}	
}


function PerformAction(params)
{
	this.Completed = true;
	this.Action = params.Action;
	this.Return = [this.Action, params.Targets];
}

function ActionTargetSelect(action)
{
	this.Completed = false;
	this.Canceled = false;
	this.Action = action;
	this.Selector = Screen.Attach(new TargetSelector(action.AllowAll));
	this.Update = function()
	{
		if (this.Selector.Completed)
		{
			if (this.Selector.Canceled)
			{
				this.Canceled = true;
			}
			else
			{
				this.Return = [this.Action, this.Selector.Targets];
			}
			this.Completed = true;
			Screen.Detach(this.Selector);
		}
	}
	this.CleanDetach = function()
	{
		Screen.Detach(this.Selector);
	}
	this.Key = function(k) { this.Selector.Key(k); }
}

function __DOFIGHT()
{
	return ['FIGHT', Battle.CurrentBattle.SelectTarget(true)];
}
function __DONOTHING()
{
	return ['NOTHING'];
}