var Scene = {
	
	Run: function(script)
	{
		EvaluateScript("scene/" + script);
	},
	
	FacePlayer: function(person)
	{
		if (!person) person = GetCurrentPerson();
		SetPersonDirection(person,
			{'north': 'south',
			 'south': 'north',
			 'east': 'west',
			 'west': 'east'}[GetPersonDirection("hero")]);
	},
	
	MovePerson: function(person, dirs, wait)
	{
		var list = dirs.split(" ");
		for (var i = 0; i < list.length; i++)
		{
			var mf = 1;
			while (list[i].charAt(list[i].length-1) == 'h')	// halve the movement
			{
				mf *= 0.5;
				list[i] = list[i].substr(0,list[i].length-1);
			}
			if (list[i].charAt(0) == 'f')
			{
				QueuePersonCommand(person, this._GetCommandForDirection(list[i])[0], true);
			}
			else if (list[i].length == 2)
			{
				var movex = 16 / GetPersonSpeedX(person) * mf;
				var movey = 16 / GetPersonSpeedY(person) * mf;
				for (var j = 0; j < Math.min(movex, movey); j++)
					for (var k = 0, dir = this._GetCommandForDirection(list[i]); k < dir.length; k++)
						QueuePersonCommand(person, dir[k], (!i && !j ? true : false));
			}
			else
			{
				var tm = 16 / (this._IsDirectionHorizontal(list[i]) ? GetPersonSpeedX(person) : GetPersonSpeedY(person)) * mf;
				for (var j = 0; j < tm; j++)
					QueuePersonCommand(person, this._GetCommandForDirection(list[i])[0], (!i && !j ? true : false));
			}
		}
		if (wait)
		{
			while (!IsCommandQueueEmpty(person))
			{
				Screen.Frame();
			}
		}
	},
	
	Wait: function(length)
	{
		for (var until = GetTime() + length; until > GetTime(); Screen.Frame())
		{
			// done in loop
		}
	},
	
	Mask: function(endMask, time)
	{
		Screen.Mask(endMask, time);
	},
	
	PlayPersonAnimation: function(person, direction)
	{
		var revert = GetPersonFrameRevert(person);
		SetPersonFrameRevert(person, 0);
		var spriteset = GetPersonSpriteset(person);
		var sdirection = this._FindSpritesetDirection(spriteset, direction);
		SetPersonDirection(person, direction);
		for (var i = 0; i < sdirection.frames.length; i++)
		{
			SetPersonFrame(person, i);
			Screen.Frames(sdirection.frames[i].delay);
		}
		SetPersonFrameRevert(person, revert);
	},
	
	PanCamera: function(endx, endy, time)
	{
		if (IsCameraAttached())
		{
			var oldAttach = GetCameraPerson();
			DetachCamera();
		}
		else
		{
			var oldAttach = false;
		}
		var currentx = GetCameraX();
		var currenty = GetCameraY();
		var difx = endx - currentx;
		var dify = endy - currenty;
		var fx = difx / time;
		var fy = dify / time;
		for (var frames = 0; frames < time; frames += Screen.Frame())
		{
			SetCameraX(currentx + fx * frames);
			SetCameraY(currenty + fy * frames);
			//Screen.Debugger.Text = GetCameraX() + "," + GetCameraY() + "/" + fx + "," + fy;
		}
		SetCameraX(endx);
		SetCameraY(endy);
		if (oldAttach)
			AttachCamera(oldAttach);
	},
	
	Message: function(text)
	{
		var message = new Message(text);
		Screen.Attach(message);
		Screen.Input(message);
		while (!message.Completed)
		{
			Screen.Frame();
		}
		Screen.Detach(message);
	},
	
	Choice: function(text, choices)
	{
		var message = new Message(text);
		var choice = new ChoiceMenu(320 - 108, 100, 96, 0);
		choice.DrawMode = Window.Draw;
		for (var i = 0; i < choices.length; i++)
		{
			choice.AddItem(choices[i], true);
		}
		choice.FigureHeight();
		Screen.Attach(message);
		Screen.Attach(choice);
		choice.Init();
		Screen.Input(choice);
		while (!choice.Completed)
		{
			Screen.Frame();
		}
		Screen.Detach(choice);
		Screen.Detach(message);
		return choice.Selected;
	},
	
	_PageLines: function(lines, perpage)
	{
		var pages = [];
		for (var i = 0; i < lines.length; i++)
		{
			if (!(i % perpage)) pages.push([]);
			pages[pages.length-1].push(lines[i]);
		}
		return pages;
	},
	
	_WrapText: function(text, width, perpage)	// this was modified (stolen) from tfwi. thanks wip ;)
	{
		if (!perpage) perpage = 4;
		lines = [];
		line = "";
		w = 0;
		for (var i = 0; i < text.length; i++)
		{
			switch (c = text.charAt(i))
			{
				case '\n':
					lines.push(line);
					line = "";
					w = 0;
					break;
				case '|':	// new page, just adds blank lines to previous page until we reach 4 lines
					lines.push(line);
					while (lines.length % perpage)
					{
						lines.push("");
					}
					line = "";
					w = 0;
					break;
				default:
					w += Font.Message.getStringWidth(c);
					if (w > width)
					{
						lines.push(line.substr(0, line.lastIndexOf(' ')));
						line = line.substr(line.lastIndexOf(' ') + 1) + c;
						w = Font.Message.getStringWidth(line);
					}
					else
					{
						line += c;
					}
			}
		}
		if (line != '') lines.push(line);
		return lines;
	},
	
	_FindSpritesetDirection: function(spriteset, direction)
	{
		for (var i = 0; i < spriteset.directions.length; i++)
		{
			if (spriteset.directions[i].name == direction)
				return spriteset.directions[i];
		}
		return false;
	},
	
	_GetCommandForDirection: function(dir)
	{
		switch (dir.toLowerCase())
		{
			case 'n': return [COMMAND_MOVE_NORTH];
			case 's': return [COMMAND_MOVE_SOUTH];
			case 'e': return [COMMAND_MOVE_EAST];
			case 'w': return [COMMAND_MOVE_WEST];
			case 'ne': return [COMMAND_MOVE_NORTH, COMMAND_MOVE_EAST];
			case 'se': return [COMMAND_MOVE_SOUTH, COMMAND_MOVE_EAST];
			case 'sw': return [COMMAND_MOVE_SOUTH, COMMAND_MOVE_WEST];
			case 'nw': return [COMMAND_MOVE_NORTH, COMMAND_MOVE_WEST];
			case 'fn': return [COMMAND_FACE_NORTH];
			case 'fs': return [COMMAND_FACE_SOUTH];
			case 'fe': return [COMMAND_FACE_EAST];
			case 'fw': return [COMMAND_FACE_WEST];
			case 'p': return [COMMAND_WAIT];
		}
		return COMMAND_WAIT;
	},
	
	_IsDirectionHorizontal: function(dir)
	{
		return 'ew'.indexOf(dir) >= 0;
	}
	
};