var Screen = {

	Objects: [],
	Inputs: [],
	PostRenderEffects: [],
	
	Debugger: new DebugMessage(),
	
	NeverRenderMap: false,
	
	StartMask: null,
	CurrentMask: CreateColor(255, 255, 255, 0),
	EndMask: null,
	
	MaskChange: false,
	MaskChangeStart: 0,
	MaskChangeEnd: 0,
	
	LastFrame: GetTime(),

	FrameSkip: false,

	Grabbed: false,
	FrameCount: 0,
	
	OldReplacedTiles: [],

	Update: function()
	{
		this.FrameCount++;
		this.LastFrame = GetTime();
		if (this.Inputs.length)
		{
			while (AreKeysLeft())
			{
				this.Inputs[this.Inputs.length - 1].Key(GetKey());
			}
		}
		var o = this.Objects.slice(0);
		var r = [];
		for (var i = 0; i < o.length; i++)
		{
			if (o[i].Update) o[i].Update();
			if (o[i].Monitor && o[i].Completed)
			{
				r.push(i);
			}
		}
		var off = 0;
		for (var i = 0; i < r.length; i++)
		{
			this.Objects.splice(r[i]-off, 1);
			off++;
		}
			
		var o = this.PostRenderEffects.slice(0);
		var r = [];
		for (var i = 0; i < o.length; i++)
		{
			if (o[i].Update) o[i].Update();
			if (o[i].Monitor && o[i].Completed)
			{
				r.push(i);
			}
		}
		var off = 0;
		for (var i = 0; i < r.length; i++)
		{
			this.PostRenderEffects.splice(r[i]-off, 1);
			off++;
		}
		if (this.MaskChange)
		{
			var factor = (GetTime() - this.MaskChangeStart);
			this.CurrentMask.red = this.StartMask.red + this.MaskChange.red * factor;
			this.CurrentMask.green = this.StartMask.green + this.MaskChange.green * factor;
			this.CurrentMask.blue = this.StartMask.blue + this.MaskChange.blue * factor;
			this.CurrentMask.alpha = this.StartMask.alpha + this.MaskChange.alpha * factor;
			if (GetTime() > this.MaskChangeEnd)
			{
				this.MaskChange = false;
				this.CurrentMask = this.EndMask;
			}
		}
		if (!this.Inputs.length) Map.UpdateMovement();
		if (Map.EnableWallHack)
		{
			if (Map.InputPerson)
			{
				for (var i = 0; i < this.OldReplacedTiles.length; i++)
				{
					SetTile(this.OldReplacedTiles[i][0][0], this.OldReplacedTiles[i][0][1], this.OldReplacedTiles[i][2], this.OldReplacedTiles[i][1]);
				}
				this.OldReplacedTiles = [];
				var layer = GetPersonLayer(Map.InputPerson);
				var x = GetPersonX(Map.InputPerson);
				var y = GetPersonY(Map.InputPerson);
				var tw = GetTileWidth();
				var th = GetTileHeight();
				var tx_s = Math.floor(x / tw);
				var ty_s = Math.floor(y / th);
				var tiles = [
					[tx_s - 1, ty_s - 1],
					[tx_s, ty_s - 1],
					[tx_s + 1, ty_s - 1],
					[tx_s - 1, ty_s],
					[tx_s, ty_s],
					[tx_s + 1, ty_s],
					[tx_s - 1, ty_s + 1],
					[tx_s, ty_s + 1],
					[tx_s + 1, ty_s + 1]];
				for (var i = 0; i < tiles.length; i++)
				{
					var oldindex = GetTile(tiles[i][0], tiles[i][1], layer+1);
					var name = GetTileName(oldindex);
					if (name.substr(0,2) == "r_")
					{
						this.OldReplacedTiles.push([tiles[i], oldindex, layer+1]);
						var newtile = parseInt(name.substr(2));
						SetTile(tiles[i][0], tiles[i][1], layer+1, newtile);
					}
				}
			}
		}
	},
	
	SetInputPerson: function(person)
	{
		this.InputPerson = person;
	},
	
	Mask: function(endMask, time)
	{
		if (!time)
		{
			this.CurrentMask = endMask;
			this.MaskChange = false;
			return;
		}
		this.StartMask = {
			red: this.CurrentMask.red,
			green: this.CurrentMask.green,
			blue: this.CurrentMask.blue,
			alpha: this.CurrentMask.alpha
		};
		
		this.MaskChange = {
			red: (endMask.red - this.StartMask.red) / time,
			green: (endMask.green - this.StartMask.green) / time,
			blue: (endMask.blue - this.StartMask.blue) / time,
			alpha: (endMask.alpha - this.StartMask.alpha) / time
		};
		this.MaskChangeStart = GetTime();
		this.MaskChangeEnd = GetTime() + time;
		this.EndMask = endMask;
	},

	
	RenderMap: function()
	{
		for (var i = 0; i < this.Objects.length; i++)
		{
			if (this.Objects[i].Render) this.Objects[i].Render();
		}
		if (this.CurrentMask.alpha)
		{
			Rectangle(0, 0, 320, 240, this.CurrentMask);
		}
	},
	
	Render: function(noMap)
	{
		if (!this.NeverRenderMap && IsMapEngineRunning()) RenderMap();
		else
		{
			for (var i = 0; i < this.Objects.length; i++)
			{
				if (this.Objects[i].Render) this.Objects[i].Render();
			}
			if (this.CurrentMask.alpha)
			{
				Rectangle(0, 0, 320, 240, this.CurrentMask);
			}
		}
	},
	
	PostRender: function()
	{
		/*if (this.FrameCount % 2 == 0 && !this.Grabbed)
		{
			var surf = GrabSurface(0, 0, 320, 240);
			var i = new String(Math.floor(this.FrameCount / 8));
			while (i.length < 4) i = "0" + i;
			surf.save("frame-" + i + ".png");
			delete surf;
			this.Grabbed = true;
		}
		else
		{
			this.Grabbed = false;
		}*/
		for (var i = 0; i < this.PostRenderEffects.length; i++)
		{
			this.PostRenderEffects[i].Render();
		}
		this.Debugger.Render();
	},
	
	Attach: function(o)
	{
		this.Objects.push(o);
		return o;
	},
	
	AttachPostRenderEffect: function(o)
	{
		this.PostRenderEffects.push(o);
		return o;
	},
	
	DetachPostRenderEffect: function(o)
	{
		for (var i = 0; i < Screen.PostRenderEffects.length; i++)
		{
			if (Screen.PostRenderEffects[i] == o)
			{
				Screen.PostRenderEffects.splice(i, 1);
				i--;
			}
		}
	},
	
	Input: function(o)
	{
		this.Inputs.push(o);
	},
	
	Detach: function(o)
	{
		var wasd = false;
		for (var i = 0; i < Screen.Objects.length; i++)
		{
			if (Screen.Objects[i] == o)
			{
				Screen.Objects.splice(i, 1);
				wasd = true;
				i--;
			}
		}
		for (var i = 0; i < Screen.Inputs.length; i++)
		{
			if (Screen.Inputs[i] == o)
			{
				Screen.Inputs.splice(i, 1);
				wasd = true;
				i--;
			}
		}
		if (o.CleanDetach) o.CleanDetach();		
		return wasd;
	},
	
	HasInput: function(o)
	{
		return this.Inputs[this.Inputs.length - 1] == o;
	},
	
	Frame: function(noFlip)
	{
		var ret = false;
		if (this.LastFrame + 16 < GetTime())
		{
			if (this.FrameSkip)
			{
				for (var i = 0; i < Math.floor((GetTime() - this.LastFrame) / 16); i++)
				{
					if (IsMapEngineRunning()) UpdateMapEngine();
					Screen.Update();
				}
			}
			else
			{
				if (IsMapEngineRunning()) UpdateMapEngine();
				Screen.Update();
			}
			ret = true;
		}
		Screen.Render();
		Screen.PostRender();
		if (!noFlip) FlipScreen();
		return ret;
	},

	
	Frames: function(num)
	{
		for (var i = 0; i < num; i += (this.Frame()?1:0))
		{
			// done in loop
		}
	}
};

function DebugMessage()
{
	this.Text = "";
	this.Render = function()
	{
		GetSystemFont().drawText(0, GetScreenHeight() - 10, this.Text);
	}
}